#ifndef __GUIDEMO_H__
#define __GUIDEMO_H__

#include "chessgui.h"
#include "messagebox.h"
#include "inputbox.h"
#include "buttonbox.h"
#include "chessinclude.h"
#include <stdio.h>
#include <string.h>
#include <iostream>

enum {W_ROOK=1, W_KNIGHT=2, W_BISHOP=3, W_QUEEN=4, W_KING=5, W_PAWN=6,
      B_ROOK=7, B_KNIGHT=8, B_BISHOP=9, B_QUEEN=10, B_KING=11, B_PAWN=12};

class Chess : public Gtk::Widget {

protected:
	// Class Variables ===========================================================================
	Gtk::Main *gtk_app;
	ChessGui gui;
	
	// Event-based functions =====================================================================
	void on_button_clicked(int which_button);
	//void on_mouse_button_pressed(int c, int r, int type, int button);
	void on_mouse_button_released(int c, int r, int button);
	/*void on_drag_begin(int c, int r, int hotx, int hoty);
	void on_drag_enter(int c, int r);
	void on_drag_leave(int c, int r);
	void on_drag_drop(bool sucess, int sx, int sy, int tx, int ty);*/
	int on_delete_event();

	// Convenience functions for students ========================================================
	void place_piece(int c, int r, int piece, int alpha=255); 	//places piece and sets drag info
	void clear_piece(int c, int r);					// clears a piece from layer 0
	void highlight_square(int c, int r, int color);			// highlights a square using the given color on layer 1
	void unhighlight_square(int c, int r);				// unhighlights a square on layer 1
		
	const char *get_piece_pixmap(int piece);
    
public:
	Chess();
	~Chess();

	void run(Gtk::Main &myapp, int argc, char *argv[]);
	
	// Convenience functions from the GUI ========================================================
	void top_label_set_text(const char *text);
	void bottom_label_set_text(const char *text);
	void info_area_clear();
	void info_area_put_text(const char *text,int fg=-1, int bg=-1);
	void input_box_clear();
	const char *input_box_get_text();
	void board_cell_set_pixmap  (int c, int r, const gchar * const *data, int layer=0, int alpha=255);

	void board_cell_set_pixmap  (int c, int r, const char *filename, int layer=0, int alpha=255);

	void board_cell_clear_pixmap(int c, int r, int layer=-1);  //clears cell
	void board_cell_color(int column, int row, int color,int layer=MAX_LAYER_VALUE,int alpha=255);  //color is 0xRRGGBB, in hex
	int button_box(const char *title, const char *message, const char **button_labels, int num_buttons);
	/*Used to create a message box. The box_type can be 0, 1, or 2. The different
	numbers corispond to a box with just an okay button, an okay and cancel, or a yes and no button*/
   	int message_box(const char *title, const char *message, int box_type);
   	const char* input_box(const char *title, const char *message, const char* label);

	
	// User Defined Functions ====================================================================
	
};

#endif
