#include "buttonbox.h"

#ifdef DEBUG_BUTTON_BOX
#define DEBUG
#include "messagebox.h"
#endif

using namespace SigC;

#define MIN_BUTTON_WIDTH 70
#define MESSAGE_FRAME_BORDER_WIDTH 10
/**
 *
 *
 */
void ButtonBox::on_button_clicked(int which_button){
	button_pressed.emit(which_button);
	last_button_event=which_button;
#ifdef DEBUG	
	g_print("button %d pressed\n",which_button);
#endif
	if(event_interrupt){
		event_interrupt=0;
		gtk_main_quit();
	}
}


/**
 *
 *
 */
bool ButtonBox::on_destroy_event(GdkEventAny *event){
	button_pressed.emit(BB_DISMISS);
	last_button_event=BB_DISMISS;
#ifdef DEBUG	
	g_print("User asked for dialog dismissal\n");
#endif
	if(event_interrupt){
		event_interrupt=0;
		gtk_main_quit();
	}
	return true; //should I return false?  can't remember!
	//we must stop the signal, since we've dealt with it
}

/**
 *
 *
 */
/*
bool ButtonBox::on_key_press_event(GdkEventKey *kev){
	if(kev->keyval == GDK_Escape){
		on_destroy_event(NULL);
		return true;
	}
	return false;
}
*/

/**
 *
 *
 */
ButtonBox::ButtonBox(){
	buttons=0;
	button=NULL;
	last_button_event=-1;
	event_interrupt=0;

	set_modal(true);
	
	message.show();
	message.set_justify(Gtk::JUSTIFY_CENTER);
	message.set_line_wrap(true);

	message_frame.set_border_width(10);
	message_frame.set_shadow_type(Gtk::SHADOW_NONE);
	message_frame.add(message);
	message_frame.show();
	
	buttons_vbox.pack_start(message_frame,Gtk::PACK_EXPAND_WIDGET,0);
	buttons_vbox.set_spacing(5);
	buttons_vbox.show();
	
	add(buttons_vbox);
	
	set_border_width(15);
	
	
	signal_delete_event().connect(SigC::slot(*this, &ButtonBox::on_destroy_event));

}

/**
 *
 *
 */
ButtonBox::ButtonBox(const char *title,
                       const char *_message,
                       int num_buttons,
                       const char **button_text,
                       int default_button) {
	buttons=0;
	button=NULL;
	last_button_event=-1;
	event_interrupt=0;

	set_modal(true);
	
	set_message(_message);
	
	message.set_justify(Gtk::JUSTIFY_CENTER);
	message.set_line_wrap(true);
	message.show();
	
	message_frame.set_border_width(10);
	message_frame.set_shadow_type(Gtk::SHADOW_NONE);
	message_frame.add(message);
	message_frame.show();
	
	buttons_vbox.pack_start(message_frame,Gtk::PACK_EXPAND_WIDGET,0);
	
	buttons_vbox.set_spacing(5);
	buttons_vbox.show();
	
	add(buttons_vbox);
	
	set_border_width(15);
	
	signal_delete_event().connect(SigC::slot(*this, &ButtonBox::on_destroy_event));
	
	set_buttons(num_buttons,button_text,default_button);
}


/**
 *
 *
 */
ButtonBox::~ButtonBox() {
	int x;

	if(button) {
		for(x=0;x<buttons;x++) {
			button_clicked_connection[x].disconnect();
//			button_keypress_connection[x].disconnect();
			delete button[x];
		}
		delete[] button_clicked_connection;
//		delete[] button_keypress_connection;
		delete[] button;
	}
	button=NULL;
	button_clicked_connection=NULL;
	button_keypress_connection=NULL;
}

/**
 *
 *
 */
void ButtonBox::set_buttons(int num_buttons, const char **button_text, int default_button) {  //This sets the box's type,only effects the button display
	int x;

	//first we need to disconnect and delete existing buttons, if any
	if(button) {
		for(x=0;x<buttons;x++) {
			button_clicked_connection[x].disconnect();
//			button_keypress_connection[x].disconnect();
			buttons_vbox.remove(*(button[x]));
			delete button[x];
		}
		delete[] button_clicked_connection;
//		delete[] button_keypress_connection;
		delete[] button;
	}
	button=NULL;
	button_clicked_connection=NULL;
	button_keypress_connection=NULL;
	buttons=num_buttons;
	
	//now we can create an array of button pointers for our new buttons
	//as well as connection objects so we can disconnect them later when
	//the dialog is reconfigured
	button=new Gtk::Button* [num_buttons];
	button_clicked_connection=new Connection[num_buttons];
//	button_keypress_connection=new Connection[num_buttons];
	
	//create each button
	for(x=0;x<buttons;x++) {
		button[x]=new Gtk::Button(button_text[x]);
		buttons_vbox.pack_start(*button[x],Gtk::PACK_SHRINK);
		button[x]->set_size_request(MIN_BUTTON_WIDTH,-1);
		button[x]->show();
		button_clicked_connection[x] = button[x]->signal_clicked().connect(SigC::bind(SigC::slot(*this,&ButtonBox::on_button_clicked),x+1));
//		button_keypress_connection[x]= button[x]->signal_key_press_event().connect(SigC::slot(*this,&ButtonBox::on_key_press_event));
		button[x]->set_flags(Gtk::CAN_FOCUS | Gtk::CAN_DEFAULT);
	}
	
	//set the default button
	button[default_button-1]->grab_focus();
	button[default_button-1]->grab_default();
	
}

/**
 *
 *
 */
void ButtonBox::set_title(const char *title){
	this->Window::set_title(string(title));
}	

/**
 *
 *
 */
void ButtonBox::set_message(const char *_message){ //This sets the message only
	message.set_text(string(_message));
}

/**
 *
 *
 */
int ButtonBox::process_events(){
	event_interrupt=1;
	gtk_main();
	return last_button_event;
}

/**
 *
 *
 */
int ButtonBox::get_last_button_clicked(){
	return last_button_event;
}

#ifdef DEBUG_BUTTON_BOX
void button_clicked() {
	static const char *mybuttons[]={"Ok","Cancel"};
	static const char *yesnobuttons[]={"Yes","No","Cancel"};
	ButtonBox bb;
	MessageBox mb;
	int ev;

	bb.set_title("Hi there");
	bb.set_message("Hello world");
	bb.set_buttons(2,mybuttons,1);
	bb.show();

	g_print("Button pressed.\n");
	ev=bb.process_events();
	switch(ev) {
		case 1:
			g_print("Button 1, quiting\n");
			break;
		case 2:
      	mb.set_title("Are you sure?");
      	mb.set_message("Are you sure you want to do this crazy thing?");
      	mb.set_type(MB_YESNO_BOX);
      	bb.set_sensitive(false);
      	mb.show();
      	mb.process_events();
      	if(mb.get_last_button_clicked()==MB_YES){
				;
		   }else{
		   	mb.set_title("Okay I won't");
		   	mb.set_message("Okay, I won't do it this time.  But you'll have to click on this button to continue!");
		   	mb.set_type(MB_OK_BOX);
		   	mb.process_events();
			}
			mb.hide();
			break;
	}




}

int main(int argc, char *argv[]) {
	Gtk::Main myapp(argc,argv);

	Gtk::Window window;
	Gtk::Button button("Hello world");

	window.show();
	window.add(button);
	button.show();
	button.signal_clicked().connect(SigC::slot(button_clicked));
	myapp.run();

	//myapp.run();
}
#endif
