#include "chessgui.h"
#include <glibmm/ustring.h>

using namespace SigC;

#ifndef CELL_SIZE
#define CELL_SIZE 32
#endif

#ifndef RIGHT_PANE_SIZE
#define RIGHT_PANE_SIZE 250
#endif

#ifndef SCALING_METHOD
#define SCALING_METHOD Gdk::INTERP_BILINEAR
#endif

ChessGui::ChessGui(){
	gdk_rgb_init();
	set_title("BYU - Chess - CS 240");

   main_vbox.show();
   add(main_vbox);

   hbox.show();
   main_vbox.pack_start(hbox,Gtk::PACK_EXPAND_WIDGET,0);
   hbox.set_border_width(5);

   left_pane.show();
   hbox.pack_start(left_pane,Gtk::PACK_EXPAND_WIDGET,0);

   label1.show();
   label1.set_text(string("Testing"));
   label1.set_justify(Gtk::JUSTIFY_LEFT);
   left_pane.pack_start(label1,Gtk::AttachOptions(),0);


   board_frame=new Gtk::AspectFrame(Glib::ustring(""),0.5,0.5,1,false);
   board_frame->show();
   board_frame->set_border_width(5);
   board_frame->set_shadow_type(Gtk::SHADOW_OUT);
   left_pane.pack_start(*board_frame,Gtk::PACK_EXPAND_WIDGET,0);

   board_table.resize(MAX_BOARD_ROWS,MAX_BOARD_COLUMNS);
   board_table.set_homogeneous(true);
   board_table.set_spacings(0);
   board_table.show();
   board_frame->add(board_table);

   //configure drawing areas here
   for(int c=0; c<MAX_BOARD_COLUMNS; c++ ){
      for( int r=0; r<MAX_BOARD_ROWS; r++){
         cell[c][r].set_size_request(CELL_SIZE,CELL_SIZE);
         cell[c][r].show();
         cell[c][r].set_events (Gdk::EXPOSURE_MASK
                | Gdk::LEAVE_NOTIFY_MASK
                | Gdk::BUTTON_PRESS_MASK
                | Gdk::BUTTON_RELEASE_MASK
                | Gdk::POINTER_MOTION_MASK
                | Gdk::ENTER_NOTIFY_MASK
                | Gdk::LEAVE_NOTIFY_MASK);
//                | Gdk::POINTER_MOTION_HINT_MASK);

         cell[c][r].signal_configure_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_cell_configure),c,r));
         cell[c][r].signal_expose_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_cell_expose),c,r));
         cell[c][r].signal_motion_notify_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_mouse_motion),c,r));
         cell[c][r].signal_button_release_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_mouse_button_released),c,r));
         cell[c][r].signal_button_press_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_mouse_button_pressed),c,r));
         cell[c][r].signal_scroll_event().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_mouse_scroll),c,r));
			cell[c][r].signal_drag_begin().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_cell_drag_begin),c,r));
			//For drag_motion we have to use our special bind that handles Signal6
			cell[c][r].signal_drag_motion().connect(bind(SigC::slot(*this,&ChessGui::on_cell_drag_motion),c,r));
         cell[c][r].signal_drag_end().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_cell_drag_end),c,r));
         cell[c][r].signal_drag_leave().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_cell_drag_leave),c,r));
			//For drag_motion we have to use our special bind that handles Signal6
         cell[c][r].signal_drag_drop().connect(bind(SigC::slot(*this,&ChessGui::on_cell_drag_drop),c,r));

         board_table.attach(cell[c][r],c,c+1,r,r+1);

         for(int i=0;i<MAX_LAYERS;i++){
            cell_user_visible[c][r][i]=false;
            cell_user_visible[c][r][i]=0; //merge this with the alpha value
            cell_color_alpha[c][r][i]=0;
         }
      }
   }

	chess_gui_target_entry.push_back( Gtk::TargetEntry(CHESS_GUI_DND_TARGET, 0 ,CHESS_GUI_DND_INFO));

   label2.show();
   label2.set_text(string("Testing"));
   label2.set_justify(Gtk::JUSTIFY_LEFT);
   left_pane.pack_start(label2,Gtk::AttachOptions(),0);

   separator.show();
   left_pane.pack_start(separator,Gtk::AttachOptions(),0);

   filename_entry_hbox.show();
   filename_entry_hbox.set_border_width(6);
   filename_entry_hbox.set_spacing(5);
   left_pane.pack_start(filename_entry_hbox,Gtk::AttachOptions(),5);

   filename_entry_label.set_text(string(DEFAULT_FILENAME_ENTRY_LABEL));
   filename_entry_label.show();
   filename_entry_hbox.pack_start(filename_entry_label,Gtk::AttachOptions(),0);

   filename_entry_box.set_text(string(""));
   filename_entry_box.show();
   filename_entry_hbox.pack_start(filename_entry_box,Gtk::PACK_EXPAND_WIDGET,0);

   right_pane.show();
   right_pane.set_size_request(RIGHT_PANE_SIZE,-1); //??-2??
   hbox.pack_start(right_pane,Gtk::AttachOptions(),0);

   button_frame.show();
   button_frame.set_shadow_type(Gtk::SHADOW_NONE);
   right_pane.pack_start(button_frame,Gtk::AttachOptions(),0);

   button_vbox.show();
   button_vbox.set_spacing(5);
   button_vbox.set_border_width(20);
   button_frame.add(button_vbox);

   for(int x=0;x<MAX_BUTTONS;x++){
      button[x].show();
      button_vbox.pack_start(button[x],Gtk::AttachOptions(),0);
      button[x].signal_clicked().connect(SigC::bind(SigC::slot(*this,&ChessGui::on_button_clicked),x));
   }
   button[0].set_label(Glib::ustring("New Game"));
   button[1].set_label(Glib::ustring("Load Game"));
   button[2].set_label(Glib::ustring("Save Game"));
   button[3].set_label(Glib::ustring("Undo Move"));

   info_frame.set_label(string(" History and Messages "));
   info_frame.show();
   info_frame.set_border_width(5);
   right_pane.pack_start(info_frame,Gtk::PACK_EXPAND_WIDGET,0);

   info_scrolled_window.show();
   info_scrolled_window.set_border_width(5);
   info_scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	info_scrolled_window.set_shadow_type(Gtk::SHADOW_IN);
	info_frame.add(info_scrolled_window);

	info_text_buffer=Gtk::TextBuffer::create();
	info_text_view.set_buffer(info_text_buffer);
	info_text_view.set_editable(false);
	info_text_view.set_cursor_visible(true);
	info_text_view.set_wrap_mode(Gtk::WRAP_WORD);
	info_text_view.set_pixels_above_lines(2);
	info_text_view.set_pixels_below_lines(2);
	info_scrolled_window.add(info_text_view);
	info_text_view.show();
	//set up some tags
	//
	
//   info_text.show();
//   info_scrolled_window.add(info_text);

   status_bar.show();

   signal_delete_event().connect(SigC::slot(*this,&ChessGui::on_delete_event));

   main_vbox.pack_start(status_bar,Gtk::AttachOptions(),0);



}

ChessGui::~ChessGui(){

	delete board_frame;
   //cleanup pixmap backing stores here

}

void ChessGui::on_button_clicked(int which_one){
//   cout << "You pressed button " << which_one << endl;
   //emit signal
   button_clicked.emit(which_one);
}

bool ChessGui::on_mouse_motion(GdkEventMotion *motion,int c, int r){

   //emit signal??

   return true;
}

bool ChessGui::on_mouse_button_released(GdkEventButton *button, int c, int r){

   //simply emit signal, that's all.
   mouse_button_released.emit(c,r,button->button);
   return true;
}

bool ChessGui::on_mouse_button_pressed(GdkEventButton *button, int c, int r){
   int type;

	//make a note of where we clicked the cell, should a drag start.
	//that way we can attach something to the cursor at the right
	//hotspot.
   hot_x=(int)button->x;
   hot_y=(int)button->y;
   //simply emit signal, that's all.
   mouse_button_pressed.emit(c,r,type,button->button);

   return false;
}

bool ChessGui::on_mouse_scroll(GdkEventScroll *scroll, int c, int r) {
	int button;
	
	if(scroll->direction == GDK_SCROLL_UP) button=4; else button=5;
	scroll_event.type       = GDK_BUTTON_PRESS;
	scroll_event.window     = scroll->window;
	scroll_event.send_event = scroll->send_event;
	scroll_event.time       = scroll->time;
	scroll_event.x          = scroll->x;
	scroll_event.y          = scroll->y;
	scroll_event.axes       = NULL;
	scroll_event.state      = scroll->state;
	scroll_event.button     = button;
	scroll_event.device     = scroll->device;
	scroll_event.x_root     = scroll->x_root;
	scroll_event.y_root     = scroll->y_root;

	return on_mouse_button_pressed(&scroll_event,c,r);
}

bool ChessGui::on_cell_expose(GdkEventExpose *event, int c, int r){
	int width,height;
	Glib::RefPtr<Gdk::GC> my_gc;

//   cout << "expose " << c << "," << r << endl;

   my_gc=cell[c][r].get_style()->get_fg_gc(cell[c][r].get_state());
	cell[c][r].get_window()->get_size(width,height);
	
	cell_backing_store[c][r]->render_to_drawable(cell[c][r].get_window(),
			                              my_gc,0,0,0,0,
													width,
													height,
													Gdk::RGB_DITHER_NORMAL,0,0);

	return false;  //continue signal propagation
}

bool ChessGui::on_cell_configure(GdkEventConfigure *ev, int c, int r){
	Glib::RefPtr<Gdk::GC> my_gc;
	Gdk::Color my_color;
   int i,x,y;
	int width,height;
	int width1,height1;
	Glib::RefPtr<Gdk::Pixbuf> scaled;
	guint32 color;
	guchar *colordata;
	
	cell[c][r].get_window()->get_size(width,height);	

	cell_backing_store[c][r]=Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB,false,8,width,height);
	cell_backing_store[c][r]->fill(0); //set to black.  (no real reason)	

//	g_print("new size: %d, %d\n",width,height);
	
   for(i=MAX_LAYERS-1;i>=0;i--){
      if(cell_user_pixmap[c][r][i] && cell_user_visible[c][r][i]){
//			if(i==MAX_LAYERS-1) {
//				cell_backing_store[c][r]=cell_user_pixmap[c][r][i]->scale_simple(width,height,Gdk::INTERP_NEAREST);
//			}else {
				scaled=cell_user_pixmap[c][r][i]->scale_simple(width,height,Gdk::INTERP_NEAREST);
				scaled->composite(cell_backing_store[c][r],
			                  0,0,width,height,
									0,0,1,1,
									SCALING_METHOD,
									cell_user_pixmap_alpha[c][r][i]);
//			}
      }

		if(cell_color_alpha[c][r][i]) {
			colordata=(guchar *)&color;
			colordata[3]  = (cell_color[c][r][i] & 0x00ff0000) >> 16; //red
			colordata[2]= (cell_color[c][r][i] & 0x0000ff00) >> 8;  //green
			colordata[1]= (cell_color[c][r][i] & 0x000000ff);       //blue
//			colordata[3]=cell_color_alpha[c][r][i]; //alpha
			colordata[0] = 0;
//			g_print("color is %0x\n",color);
			scaled=Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB,false,8,width,height);
			scaled->fill(color);
			scaled->composite(cell_backing_store[c][r],
			                  0,0,width,height,
									0,0,1,1,
									SCALING_METHOD,
									cell_color_alpha[c][r][i]);

		}
   }

   if(c==0 && r==0){
		cell[c][r].get_window()->get_size(width,height);	
		cell_resized.emit(width,height);
   }

   return false;
}

void ChessGui::on_cell_drag_begin(const Glib::RefPtr<Gdk::DragContext> &context, int c, int r){

   int hotx, hoty;

   drag_success=true;  //should be drag_in_progress

   drag_source_c=c;
   drag_source_r=r;

   drag_target_c=drag_source_c;
   drag_target_r=drag_source_r;

   drag_context=context;

   cell_drag_begin.emit(c,r,hot_x, hot_y);

}

void ChessGui::on_cell_drag_end  (const Glib::RefPtr<Gdk::DragContext> &context, int c, int r){

//   cout << "Drag end; synthesizing drag drop event, if it hasn't occurred" << endl;
	if(drag_source_c == drag_target_c && drag_source_r == drag_target_r){
      cell_drag_drop.emit(false,drag_source_c,drag_source_r,drag_target_c,drag_target_r);
   }else{
      cell_drag_drop.emit(true,drag_source_c,drag_source_r,drag_target_c,drag_target_r);
   }

}

void ChessGui::on_cell_drag_leave (const Glib::RefPtr<Gdk::DragContext> &context, guint, int c, int r){
//   cout << "Drag leave" << endl;

	//when leaving a cell, we don't know if the drop will be valid, so we'll
	//mark it as invalid until motion changes that.
	drag_target_c=drag_source_c;
   drag_target_r=drag_source_r;

   //emit signal
   cell_drag_leave.emit(c,r);

}

//void     on_cell_drag_enter (GdkDragContext *, int c, int r); //not needed for dnd to work

bool ChessGui::on_cell_drag_motion(const Glib::RefPtr<Gdk::DragContext> &context, int x, int y, guint time, int c, int r){
//   cout << "Motion in " << c << ", " << r << endl;
   //emit signal
   if(c != drag_target_c || r != drag_target_r){
      cell_drag_enter.emit(c,r);
   }
   cell_drag_motion.emit(c,r);
   drag_target_c=c;
   drag_target_r=r;

   return false;
}

bool ChessGui::on_cell_drag_drop (const Glib::RefPtr<Gdk::DragContext> &context, int x, int y, guint time, int c, int r){
//   cout << "Drop message" << endl;
   drag_target_c=c;
   drag_target_r=r;
//   on_cell_drag_end (NULL,c,r);
   return true;

}

bool ChessGui::on_delete_event(GdkEventAny*){
   return main_window_close.emit();
}

void ChessGui::main_window_set_size(int width, int height){
   set_size_request(width,height);
}

void ChessGui::main_window_set_title(const char *text){
   set_title(text);
}

void ChessGui::main_window_set_active(bool active){
   set_sensitive(active);
}

void ChessGui::main_window_hide(){
   hide();
}

void ChessGui::main_window_show(){
   show();
}


void ChessGui::input_box_clear() {
   filename_entry_box.set_text(string(""));
}

void ChessGui::input_box_set_active(bool active) {
   filename_entry_box.set_sensitive(active);
}

const char *ChessGui::input_box_get_text(){ //maybe make this use the stl string class (and all text)
   return filename_entry_box.get_text().c_str();  //this is not kosher!!!
}

void ChessGui::status_bar_push_text(const char *text){

}

void ChessGui::status_bar_pop_text(){
}

void ChessGui::status_bar_set_text(const char *text){
}

void ChessGui::top_label_set_text(const char *text){
   label1.set_text(text);
}

void ChessGui::bottom_label_set_text(const char *text){
   label2.set_text(text);
}


void ChessGui::button_set_active(int which_button, bool active){
   if(which_button>=MAX_BUTTONS) return;

   button[which_button].set_sensitive(active);

}


void ChessGui::board_cell_color(int c, int r, int color, int layer, int alpha){  //color is 0xRRGGBB, in hex

	cell_color[c][r][layer]=color;
	cell_color_alpha[c][r][layer]=alpha;
	//g_print("setting color at %d,%d,%d to %d, alpha %d\n",c,r,layer,color, alpha);
   on_cell_configure(NULL,c,r);
   cell[c][r].queue_draw();
}

void ChessGui::info_area_clear(){
	//info_text.delete_text(0,info_text.get_length());
   //????
//	info_text_buffer->delete_text(info_text_buffer->begin(),info_text_buffer->end());
	info_text_buffer->erase(info_text_buffer->begin(),info_text_buffer->end());
}

void ChessGui::info_area_put_text(const char *text,int _fg, int _bg){
/*
	Glib::RefPtr<Gdk::GC> my_gc;
	Gdk::Color fg;
	Gdk::Color bg;
//   Gdk_Font font;

   my_gc=info_text.get_style()->get_text_gc(info_text.get_state());

   if(_fg>=0){
      fg.set_red  ((_fg & 0x00ff0000) >> 8 | 0x000000ff);
      fg.set_green( _fg & 0x0000ff00 | 0x000000ff);
      fg.set_blue ((_fg & 0x000000ff) << 8 | 0x000000ff);
   }
   if(_bg>=0){
      bg.set_red  ((_bg & 0x00ff0000) >> 8 | 0x000000ff);
      bg.set_green( _bg & 0x0000ff00 | 0x000000ff);
      bg.set_blue ((_bg & 0x000000ff) << 8 | 0x000000ff);
   }
   if(_fg>=0 && _bg >=0){
      info_text.insert(my_gc.get_font(),fg,bg,Gtk::nstring(text),strlen(text));
   }else{
      info_text.insert(string(text));
   }
	*/
	info_text_buffer->insert_at_cursor(Glib::ustring(text));
	info_text_view.scroll_to_mark(info_text_buffer->get_insert(),0);
}

void ChessGui::info_area_put_text(const char *text, const char *font, int _fg, int _bg){

/*
   Gdk_GC my_gc;
   Gdk_Color fg;
   Gdk_Color bg;
   Gdk_Font my_font;

   my_font=Gdk_Font(string(font));

   my_gc=info_text.get_style()->get_text_gc(info_text.get_state());

   if(_fg>=0){
      fg.set_red  ((_fg & 0x00ff0000) >> 8 | 0x000000ff);
      fg.set_green( _fg & 0x0000ff00 | 0x000000ff);
      fg.set_blue ((_fg & 0x000000ff) << 8 | 0x000000ff);
   }
   if(_bg>=0){
      bg.set_red  ((_bg & 0x00ff0000) >> 8 | 0x000000ff);
      bg.set_green( _bg & 0x0000ff00 | 0x000000ff);
      bg.set_blue ((_bg & 0x000000ff) << 8 | 0x000000ff);
   }

   info_text.insert(my_font,fg,bg,Gtk::nstring(text),strlen(text));
*/
	info_text_buffer->insert_at_cursor(Glib::ustring(text));
	info_text_view.scroll_to_mark(info_text_buffer->get_insert(),0);
}

void ChessGui::board_cell_set_pixmap(int c, int r, const gchar * const *data, int layer, int alpha){

   if(layer >= MAX_LAYERS) return;

	cell_user_pixmap[c][r][layer]=Gdk::Pixbuf::create_from_xpm_data ((const char **)data);
   cell_user_pixmap_alpha[c][r][layer]=alpha;
   cell_user_visible[c][r][layer]=true;

   on_cell_configure(NULL,c,r);
   cell[c][r].queue_draw();

}

void ChessGui::board_cell_set_pixmap(int c, int r, const char *file, int layer, int alpha){
	//   g_print("Trying to load %s\n",file);
	if(layer >= MAX_LAYERS) return;

	cell_user_pixmap[c][r][layer]=Gdk::Pixbuf::create_from_file (file);
	cell_user_pixmap_alpha[c][r][layer]=alpha;
	cell_user_visible[c][r][layer]=true;
	on_cell_configure(NULL,c,r);
	cell[c][r].queue_draw();
}

void ChessGui::board_cell_clear_pixmap(int c, int r, int layer){  //clears cell

   if(layer >= MAX_LAYERS) return;

   if(layer<0){
      for(int i=0;i<MAX_LAYERS;i++){
         cell_user_visible[c][r][i]=false; 
      }
   }else{
      cell_user_visible[c][r][layer]=false;
   }

   on_cell_configure(NULL,c,r);
   cell[c][r].queue_draw();


}

void ChessGui::board_cell_pixmap_show(int c, int r, int layer){
   if(layer >= MAX_LAYERS) return;

   if(layer<0){
      for(int i=0;i<MAX_LAYERS;i++){
         cell_user_visible[c][r][i]=true;
      }
   }else{
      cell_user_visible[c][r][layer]=true;
   }

   on_cell_configure(NULL,c,r);
   cell[c][r].queue_draw();

}

void ChessGui::board_cell_pixmap_hide(int c, int r, int layer){

   if(layer >= MAX_LAYERS) return;

   if(layer<0){
      for(int i=0;i<MAX_LAYERS;i++){
         cell_user_visible[c][r][i]=false;
      }
   }else{
      cell_user_visible[c][r][layer]=false;
   }

// cout << "cp1" <<endl;
   on_cell_configure(NULL,c,r);
   cell[c][r].queue_draw();
// cout << "cp2" <<endl;

}

int  ChessGui::board_cell_get_width(int c, int r){
	int width,height;

	cell[c][r].get_window()->get_size(width,height);
	return width;
}

int  ChessGui::board_cell_get_height(int c, int r){
	int width,height;

	cell[c][r].get_window()->get_size(width,height);
	return height;
}

void ChessGui::board_cell_set_drag_source(int c, int r){
   cell[c][r].drag_source_set(chess_gui_target_entry,
			                     Gdk::BUTTON1_MASK,
										Gdk::ACTION_COPY);

}

void ChessGui::board_cell_unset_drag_source(int c, int r){
   cell[c][r].drag_source_unset();
}

void ChessGui::board_cell_set_drag_dest(int c, int r){
   cell[c][r].drag_dest_set(chess_gui_target_entry,
									 Gtk::DEST_DEFAULT_ALL,
									 Gdk::ACTION_COPY );
}

void ChessGui::board_cell_unset_drag_dest(int c, int r){
   cell[c][r].drag_dest_unset();
}

void ChessGui::board_cell_set_drag_cursor(int c, int r, int hot_spot_x, int hot_spot_y, const gchar * const *pixmap,int width, int height){
	Glib::RefPtr<Gdk::Colormap> cm;

   int hotx, hoty;

//   if(p) p.release();
//   if(b) b.release();
   cm=get_window()->get_colormap();
   p=Gdk::Pixmap::create_from_xpm(get_window()->get_colormap(),b,pixmap);


	int pwidth,pheight;

	p->get_size(pwidth,pheight);
	g_print("%d, %d\n",width,height);
	g_print("%d, %d\n",pwidth,pheight);

   if(width && height) {
	   //Since we can't really scale, adjust the hotpots as if we were
	   //scaling...
		hotx=hot_spot_x-(width-pwidth)/2;
		hoty=hot_spot_y-(height-pheight)/2;
   } else {
		//hotspots are not adjusted.
		hotx=hot_spot_x;
		hoty=hot_spot_y;
   }

	g_print("%d, %d\n",hotx,hoty);
	

//	cell[c][r].drag_set_as_icon(drag_context,hotx,hoty);
//	cell[c][r].drag_source_set_icon(cm,p,b);
	drag_context->set_icon(cm,p,b,hotx,hoty);

}

void ChessGui::board_cell_set_drag_cursor(int c, int r, int hot_spot_x, int hot_spot_y, const char *file,int width, int height){

	Glib::RefPtr<Gdk::Pixbuf> scaled;

   scaled=Gdk::Pixbuf::create_from_file(file);
	
   pb=scaled->scale_simple(width,height,SCALING_METHOD);
	drag_context->set_icon(pb,hot_spot_x,hot_spot_y);

}

//not needed, as gtkmm releases everything when drag finishes.
void ChessGui::board_cell_unset_drag_cursor(int hot_spot_x, int hot_spot_y, const gchar * const *pixmap){

}

int ChessGui::button_box(const char *title, const char *message, const char **button_labels, int num_buttons) {
	int button_clicked;
	ButtonBox *buttons = new ButtonBox();
	main_window_set_active(false);
	buttons->set_title(title);
	buttons->set_message(message);
	buttons->set_buttons(num_buttons, button_labels);
	buttons->show();
	button_clicked = buttons->process_events();
	buttons->hide();
	main_window_set_active(true);
      delete buttons;
	return button_clicked;
} // button_box

int ChessGui::message_box(const char *title, const char *message, int box_type) {
	int button_clicked;
	MessageBox *messbox = new MessageBox();
	main_window_set_active(false);
	messbox->set_title(title);
	messbox->set_message(message);
	messbox->set_type(box_type);
	messbox->show();
	button_clicked = messbox->process_events();
	messbox->hide();
	main_window_set_active(true);
	delete messbox;
	return button_clicked;
} // message_box

// input_box opens a dialog that allows a user to input text. It returns the text they input.
const char *ChessGui::input_box(const char *title, const char *message, const char* label) {
	InputBox *inbox = new InputBox();
	main_window_set_active(false);
	inbox->set_title(title);
	inbox->set_message(message);
	inbox->set_input_label(label);
	inbox->show();
	inbox->process_events();
	const char *input = inbox->get_input_text();
	inbox->hide();
	main_window_set_active(true);
	delete inbox;
	return input;
} // input_box
