#ifndef __CHESSGUI_H__
#define __CHESSGUI_H__

#include <sigc++/sigc++.h>
#include <list>
#ifdef SIGC_CXX_NAMESPACES
using namespace std;
using namespace SigC;
#endif


#include <gtkmm/base.h>
#include <gtkmm/main.h>
#include <gtkmm/style.h>
#include <gtkmm/window.h>
#include <gtkmm/button.h>
#include <gtkmm/box.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/entry.h>
#include <gtkmm/frame.h>
#include <gtkmm/table.h>
#include <gtkmm/textview.h>
#include <gtkmm/textbuffer.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/separator.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/aspectframe.h>

#include <gdkmm/dragcontext.h>
#include <gdkmm/pixmap.h>
#include <gdkmm/bitmap.h>

#include <gdkmm/pixbuf.h>

#define MAX_BUTTONS       4
#define MAX_BOARD_COLUMNS 8
#define MAX_BOARD_ROWS    8
#define MAX_LAYERS        5
#define MAX_LAYER_VALUE   4
#define DEFAULT_FILENAME_ENTRY_LABEL "Filename:"

#define CHESS_GUI_DND_TARGET "//byuchessgui//"
#define CHESS_GUI_DND_INFO 0xdeadbeef
#include "buttonbox.h"
#include "messagebox.h"
#include "inputbox.h"

/****
 * Because the libsigc++ library doesn't have any build-in bind to bind 2 params to a Signal4,
 * we make one ourselves.  See <sigc++/bind.h>
 *
 **/
template <class R,class P1,class P2,class P3,class P4,class C1,class C2>
struct AdaptorBindSlot4_2_
  { 
    typedef typename Trait<R>::type RType;
    typedef typename Slot6<R,P1,P2,P3,P4,C1,C2>::Proxy Proxy; 
    static RType proxy(typename Trait<P1>::ref p1,typename Trait<P2>::ref p2,typename Trait<P3>::ref p3,typename Trait<P4>::ref p4, void *data)
      {
        typedef AdaptorBindData2_<C1,C2> Data;
        Data& node=*reinterpret_cast<Data*>(data);
        SlotNode* slot=static_cast<SlotNode*>(node.adaptor.slot_.impl());
        return ((Proxy)(slot->proxy_))
          (p1,p2,p3,p4,node.c1_,node.c2_,slot);
      }
  
  };

template <class A1,class A2,class R,class P1,class P2,class P3,class P4,class C1,class C2>
Slot4<R,P1,P2,P3,P4>
  bind(const Slot6<R,P1,P2,P3,P4,C1,C2>& s,
       A1 a1,A2 a2)
  { 
    typedef AdaptorBindData2_<C1,C2> Data;
    typedef AdaptorBindSlot4_2_<R,P1,P2,P3,P4,C1,C2> Adaptor;
    return reinterpret_cast<SlotNode*>(
       new Data((FuncPtr)(&Adaptor::proxy),s,
                (FuncPtr)(&Data::dtor),a1,a2));
  }



//enum {LOAD_BUTTON, SAVE_BUTTON, UNDO_BUTTON, DEBUG_BUTTON} GUI_BUTTONS;

class ChessGui:public Gtk::Window {
protected:

	//internal private variables and gtk+ widgets
   bool drag_success;
   int  drag_target_c;
   int  drag_target_r;
   int  drag_source_c;
   int  drag_source_r;
   int hot_x;
   int hot_y;
	GdkEventButton scroll_event; //temporary space for a special synthesized event.

	Glib::RefPtr<Gdk::DragContext> drag_context;

	Glib::RefPtr<Gdk::Pixmap> p;
	Glib::RefPtr<Gdk::Bitmap> b;
	Glib::RefPtr<Gdk::Pixbuf> pb;
	
	std::list<Gtk::TargetEntry> chess_gui_target_entry;
//	Gtk::TargetEntry chess_gui_target_entry[1];0

   Gtk::VBox main_vbox;
   Gtk::Statusbar status_bar;
   Gtk::HBox hbox;

   Gtk::VBox left_pane;
   Gtk::Label label1;  //maybe change this to a text widget for fonts, etc
   Gtk::AspectFrame *board_frame;
   Gtk::Table board_table;
//   Gtk::AspectFrame table_aspect_frame(0.5,0.5,1,false);
   Gtk::DrawingArea cell[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS];
   Glib::RefPtr<Gdk::Pixbuf> cell_backing_store[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS];

	Glib::RefPtr<Gdk::Pixbuf> cell_user_pixmap[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS][MAX_LAYERS];
	int                       cell_user_pixmap_alpha[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS][MAX_LAYERS];
   int cell_color_alpha[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS][MAX_LAYERS];

   int cell_color[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS][MAX_LAYERS];
   bool cell_user_visible[MAX_BOARD_COLUMNS][MAX_BOARD_ROWS][MAX_LAYERS];


   Gtk::Label label2;  //maybe change this to a text widget for fonts, etc
   Gtk::HSeparator separator;

   Gtk::HBox filename_entry_hbox;
   Gtk::Label filename_entry_label;
   Gtk::Entry filename_entry_box;

   Gtk::VBox right_pane;
   Gtk::Frame button_frame;
   Gtk::VBox  button_vbox;
   Gtk::Button button[4];  //initialize 4 buttons
   Gtk::Frame 							info_frame;
   Gtk::ScrolledWindow           info_scrolled_window;
//	Gtk::Frame							info_text_view_frame;
	Gtk::TextView                 info_text_view;
	Glib::RefPtr<Gtk::TextBuffer> info_text_buffer;

   //private, local event callbacks:

//   void on_button_clicked();
   void on_button_clicked(int which_button);
   bool on_mouse_motion(GdkEventMotion *,int c, int r);
   bool on_mouse_button_released(GdkEventButton *, int c, int r);
   bool on_mouse_button_pressed(GdkEventButton *, int c, int r);
	bool on_mouse_scroll(GdkEventScroll *,int c, int r);
	bool on_cell_expose(GdkEventExpose *, int c, int r);
   bool on_cell_configure(GdkEventConfigure *, int c, int r);

   void on_cell_drag_begin(const Glib::RefPtr<Gdk::DragContext> &, int c, int r);
   void on_cell_drag_end  (const Glib::RefPtr<Gdk::DragContext> &, int c, int r);
   void on_cell_drag_leave (const Glib::RefPtr<Gdk::DragContext> &, guint, int c, int r);
// void on_cell_drag_enter (const Glib::RefPtr<Gdk::DragContext> &, int c, int r);
   bool on_cell_drag_motion(const Glib::RefPtr<Gdk::DragContext> &, int, int, guint, int c, int r);
   bool on_cell_drag_drop  (const Glib::RefPtr<Gdk::DragContext> &, int, int, guint, int c, int r);
   bool on_delete_event(GdkEventAny *);


public:
   ChessGui();
   ~ChessGui();

   //signals

   //resize signals??

   //user must capture the destroy event!!!
   //signal int main_window_close ()
	Signal0<int> main_window_close;

   //signal void cell_resized (int width, int height)
   Signal2<void,int,int> cell_resized;

   //signal void button_clicked(int which_button)
   Signal1<void,int> button_clicked;

   //signal void mouse_button_released(int c, int r, int button);
   //button: 1=left, 2=right, 3=middle, etc
   Signal3<void,int,int,int> mouse_button_released;

   //signal void mouse_button_pressed(int c, int r, int type,int button);
   //type: 1=normal click, 2=double click, 3=triple click
   //button: 1=left, 2=middle, 3=right, etc
   Signal4<void,int,int,int,int> mouse_button_pressed;

   //signal void mouse_motion(int c, int r)
   Signal2<void,int,int> mouse_motion;

   //signal void drag_begin (int c, int r, int hot_x, int hot_y);
   Signal4 <void,int,int,int,int> cell_drag_begin;

   //signal void cell_drag_enter (int c, int r)  //this one is synthesized
   Signal2<void,int,int> cell_drag_enter;

   //signal void cell_drag_leave (int c, int r)
   Signal2<void,int,int> cell_drag_leave;

   //signal void cell_drag_motion (int c, int r)
   Signal2<void,int,int> cell_drag_motion;

   //signal void drag_drop (bool success, int src_x, int src_y, int dst_x, int dst_y);
   Signal5<void,bool,int,int,int,int> cell_drag_drop;

   //gui control methods
   //  change these all to string stl objects!
   void main_window_set_size(int width, int height);
   void main_window_set_title(const char *text);
   void main_window_set_active(bool active);
   void main_window_hide();  //use the inherited show() and hide() methods
   void main_window_show();

   void input_box_clear();
   void input_box_set_active(bool active);
   const char *input_box_get_text(); //maybe make this use the stl string class (and all text)

   void status_bar_push_text(const char *text);
   void status_bar_pop_text();
   void status_bar_set_text(const char *text);

   void top_label_set_text(const char *text);
   void bottom_label_set_text(const char *text);

   void button_set_active(int which_button, bool active);
//   void button_set_text(int which_button, const char *text);

   void info_area_clear();
   void info_area_put_text(const char *text,int fg=-1, int bg=-1);
   void info_area_put_text(const char *text, const char *font, int fg, int bg);

   // if gdk_pixbuf is not used, color is alays applied, solid to the background of the cell
   //    in this case subsequent tiles will paint over it.
   void board_cell_color(int column, int row, int color,int layer=MAX_LAYER_VALUE,int alpha=255);  //color is 0xRRGGBB, in hex
   void board_cell_set_pixmap  (int c, int r, const gchar * const *data, int layer=0, int alpha=255);

   void board_cell_set_pixmap  (int c, int r, const char *filename, int layer=0, int alpha=255);

   void board_cell_clear_pixmap(int c, int r, int layer=-1);  //clears cell
   void board_cell_pixmap_show(int c, int r, int layer=-1);
   void board_cell_pixmap_hide(int c, int r, int layer=-1);
   int  board_cell_get_width(int c, int r);
   int  board_cell_get_height(int c, int r);

   void board_cell_set_drag_source  (int column, int row);
   void board_cell_unset_drag_source(int column, int row);
   void board_cell_set_drag_dest  (int column, int row);
   void board_cell_unset_drag_dest(int column, int row);
   void board_cell_set_drag_cursor  (int source_c, int source_r,
                                     int hot_spot_x, int hot_spot_y,
                                     const gchar * const *pixmap,
                                     int width=0, int height=0);

   void board_cell_set_drag_cursor  (int source_c, int source_r,
                                     int hot_spot_x, int hot_spot_y,
                                     const char *pixmap,
                                     int width=0, int height=0);

	//why is this needed?
   void board_cell_unset_drag_cursor(int hot_spot_x, int hot_spot_y, const gchar * const *pixmap);
   
   int button_box(const char *title, const char *message, const char **button_labels, int num_buttons);
   int message_box(const char *title, const char *message, int box_type);
   const char* input_box(const char *title, const char *message, const char* label);

};


#endif
