#include "chess.h"


#ifndef MIN_DRAG_SIZE
#define MIN_DRAG_SIZE 64
#endif

// on_button_clicked ======================================================================
// 	Called when one of the four side buttons is pressed
//	0=New Game	1=Load Game	2=Save Game	3=Undo Move
void Chess::on_button_clicked(int which_button){

}

// on_mouse_button_released ==================================================================
//      Called when the mouse button is released over a particular square
//      button=which button was released        c=column        r=row

void Chess::on_mouse_button_released(int c, int r, int button){
	
}
// on_delete_event ==================================================================
//	This function is called when the GUI is closed.
int Chess::on_delete_event(){
   //we should do a wrapper for this in ChessGui
   exit(0);
//   return 1; //returning 1 effectively cancels the close

}

// Convenience functions from the GUI ======================================================================
// 	These functions simply call the corresponding functions inside the GUI
//	See the project specification, under the user interface, for details on these methods.
void Chess::top_label_set_text(const char *text)	{
	gui.top_label_set_text(text);
}
void Chess::bottom_label_set_text(const char *text)	{
	gui.bottom_label_set_text(text);
}
void Chess::info_area_clear()	{
	gui.info_area_clear();
}
void Chess::info_area_put_text(const char *text,int fg, int bg)	{
	gui.info_area_put_text(text,fg,bg);
}
void Chess::input_box_clear()	{
	gui.input_box_clear();
}
const char* Chess::input_box_get_text()	{
	return gui.input_box_get_text();
}
void Chess::board_cell_set_pixmap(int c, int r, const gchar * const *data, int layer, int alpha)	{
	gui.board_cell_set_pixmap(c,r,data,layer,alpha);
}
void Chess::board_cell_set_pixmap  (int c, int r, const char *filename, int layer, int alpha)	{
	gui.board_cell_set_pixmap(c,r,filename,layer,alpha);
}
void Chess::board_cell_clear_pixmap(int c, int r, int layer)	{
	gui.board_cell_clear_pixmap(c,r,layer);
}
void Chess::board_cell_color(int column, int row, int color,int layer,int alpha)	{  //color is 0xRRGGBB, in hex
	gui.board_cell_color(column,row,color,layer,alpha);
}
int Chess::button_box(const char *title, const char *message, const char **button_labels, int num_buttons)	{
	gui.button_box(title,message,button_labels,num_buttons);
	return 0;
}
int Chess::message_box(const char *title, const char *message, int box_type)	{
	gui.message_box(title,message,box_type);
	return 0;
}
const char* Chess::input_box(const char *title, const char *message, const char* label)	{
	gui.input_box(title,message,label);
	return "cs240";
}

// Convenience functions for the student =================================================================
//	these functions are provided as helper functions for the student. 

// place_piece ==================================================================
//	Places a piece on a particular square. Alpha is an optional argument, which specifys transparency.
//	See the enum at the top of chess.h for info on what piece should be.
void Chess::place_piece(int c, int r, int piece,int alpha){ 	//places piece on a particular square
	gui.board_cell_set_pixmap(c,r,get_piece_pixmap(piece),0,alpha);
	gui.board_cell_unset_drag_dest(c,r);
	gui.board_cell_set_drag_source(c,r);
}

// clear_piece ==================================================================
//	clears a piece from the foremost layer (0) of the given cell
void Chess::clear_piece(int c, int r)	{	
	gui.board_cell_clear_pixmap(c,r,0);
}

// highlight_square ==================================================================
//	Highlights a square using the given color on layer 1. Color is in hexidecimal 0xRRGGBB, or use the
//	predefined colors in chessinclude.h
void Chess::highlight_square(int c, int r, int color) {	
	if ((c & 1) && (r & 1))	// dark square
		gui.board_cell_color(c,r,color,1,130);
	else gui.board_cell_color(c,r,color,1,150);
}

// unhighlight_square ==================================================================
//	Un-highlights a square (ie sets it to black)
void Chess::unhighlight_square(int c, int r)	{
	gui.board_cell_color(c,r,0,1,0);
}

const char *Chess::get_piece_pixmap(int piece){
	switch(piece){
	case W_ROOK:   return WHITE_ROOK;
	case W_KNIGHT: return WHITE_KNIGHT;
	case W_BISHOP: return WHITE_BISHOP;
	case W_QUEEN:  return WHITE_QUEEN;
	case W_KING:   return WHITE_KING;
	case W_PAWN:   return WHITE_PAWN;
	case B_ROOK:   return BLACK_ROOK;
	case B_KNIGHT: return BLACK_KNIGHT;
	case B_BISHOP: return BLACK_BISHOP;
	case B_QUEEN:  return BLACK_QUEEN;
	case B_KING:   return BLACK_KING;
	case B_PAWN:   return BLACK_PAWN;

	}

	return BLACK_PAWN; //invalid input!!

}

// Chess Constructor =================================================================
//	place your own startup code as needed
Chess::Chess(){
	gui.main_window_show();
	
	gui.main_window_set_title("BYU CS 240 :: Chess ");
	
	gui.top_label_set_text("Hello");
	gui.bottom_label_set_text("World");
	
	gui.button_clicked.connect(SigC::slot(*this, &Chess::on_button_clicked));
	gui.mouse_button_released.connect(SigC::slot(*this,&Chess::on_mouse_button_released));
	gui.main_window_close.connect(SigC::slot(*this,&Chess::on_delete_event));
	
	// Board setup ======================================================================
	//	places the woodgrain texture on layer 2, and a red mask to layer 4
	for(int r=0;r<8;r++){
		for(int c=(r & 0x01 ? 0 : 1);c<8;c+=2){
			if(r & 0x01){
				gui.board_cell_color(c+1,r,0x00ff7f00,2,120);
				gui.board_cell_set_pixmap(c+1,r,WOOD,4);
				gui.board_cell_set_drag_dest(c+1,r);
			}else{
				gui.board_cell_color(c-1,r,0x00ff7f00,2,120);
				gui.board_cell_set_pixmap(c-1,r,WOOD,4);
				gui.board_cell_set_drag_dest(c-1,r);
			}
		gui.board_cell_set_pixmap(c,r,WOOD,4,255);
		gui.board_cell_color(c,r,0x0800000,4,130);     //applies a reddish alph mask to wood.xpm in layer 2
		gui.board_cell_set_drag_dest(c,r);
		}
	}
	
	// example for placing to black rooks.
	place_piece(0,0,B_ROOK);
	place_piece(7,0,B_ROOK);
}
  
Chess::~Chess(){
}

void Chess::run(Gtk::Main &myapp, int argc, char *argv[]){
	gtk_app=&myapp;
	myapp.run();
}

int main(int argc, char *argv[]){
	Gtk::Main myapp(argc,argv);
	Chess chess;
	
	chess.run(myapp,argc,argv);
	return 0;
}
